@@ -3,6 +3,7 @@ from django.utils.functional import update_wrapper
 from django.utils.encoding import force_unicode
 from django.core.urlresolvers import reverse
 from django.contrib.admin import widgets
+from django.contrib.admin.options import get_ul_class
 from django import forms
 
 from dockit.paginator import Paginator
@@ -226,19 +227,43 @@ class SchemaAdmin(object):
         from dockit import schema
         from fields import DotPathField
         from dockit.forms.fields import HiddenJSONField
+        
+        if isinstance(prop, schema.ModelReferenceField):
+            return self.formfield_for_foreignkey(prop, field, view, **kwargs)
+        if isinstance(prop, schema.ModelSetField):
+            return self.formfield_for_manytomany(prop, field, view, **kwargs)
+        
         request = kwargs.pop('request', None)
-        if ((isinstance(prop, schema.ListField) and isinstance(prop.subfield, schema.TypedSchemaField)) or
-             isinstance(prop, schema.TypedSchemaField)):
+        base_property = prop
+        if isinstance(prop, schema.ListField):
+            base_property = prop.subfield
+        
+        #TODO self.raw_id_fields
+        #TODO ForeignKeyRawIdWidget => DocumentReferenceRawIdWidget
+        if prop.name in self.raw_id_fields:
+            if isinstance(prop, schema.ReferenceField):
+                pass
+            elif isinstance(prop, schema.ModelReferenceField):
+                pass
+        
+        if (isinstance(base_property, schema.TypedSchemaField) or 
+             (isinstance(base_property, schema.SchemaField) and base_property.schema._meta.typed_field)):
             from fields import TypedSchemaField
             field = TypedSchemaField
             kwargs['dotpath'] = view.dotpath()
-            kwargs['schema_property'] = prop
-            if self.next_dotpath():
+            kwargs['params'] = request.GET.copy()
+            if isinstance(base_property, schema.SchemaField):
+                type_selector = base_property.schema._meta.fields[base_property.schema._meta.typed_field]
+            else:
+                type_selector = base_property
+            kwargs['schema_property'] = type_selector
+            if view.next_dotpath():
                 kwargs['required'] = False
             return field(**kwargs)
         if issubclass(field, HiddenJSONField):
             field = DotPathField
             kwargs['dotpath'] = view.dotpath()
+            kwargs['params'] = request.GET.copy()
             if view.next_dotpath():
                 kwargs['required'] = False
             return field(**kwargs)
@@ -254,6 +279,32 @@ class SchemaAdmin(object):
             kwargs = dict(opts, **kwargs)
         return field(**kwargs)
     
+    def formfield_for_foreignkey(self, prop, field, view, **kwargs):
+        """
+        Get a form Field for a ForeignKey.
+        """
+        if prop.name in self.raw_id_fields:
+            kwargs['widget'] = widgets.ForeignKeyRawIdWidget(rel=None)
+        elif prop.name in self.radio_fields:
+            kwargs['widget'] = widgets.AdminRadioSelect(attrs={
+                'class': get_ul_class(self.radio_fields[prop.name]),
+            })
+            kwargs['empty_label'] = prop.blank and _('None') or None
+
+        return prop.formfield(**kwargs)
+
+    def formfield_for_manytomany(self, prop, field, view, **kwargs):
+        """
+        Get a form Field for a ManyToManyField.
+        """
+        if prop.name in self.raw_id_fields:
+            kwargs['widget'] = widgets.ManyToManyRawIdWidget(rel=None)
+            kwargs['help_text'] = ''
+        elif prop.name in (list(self.filter_vertical) + list(self.filter_horizontal)):
+            kwargs['widget'] = widgets.FilteredSelectMultiple(prop.verbose_name, (prop.name in self.filter_vertical))
+
+        return prop.formfield(**kwargs)
+    
     def log_addition(self, request, object):
         return self.documentadmin.log_addition(request, object)
     
